unit DukeType;

interface
uses Classes, Graphics, WinTypes, WinProcs;

{Windows can only handle 16 bit coordinates, and BUILD coordinates
 are all 32 bit.  I divide every coordinate by 16 before scaling and
 displaying the level. }

const MAPDIV = 16;

type
 { The header of a MAP file, version 7 }
 TDukeHeader = record
   version: longint;
   Startx: longint;
   Starty: longint;
   Startz: longint;
   Startang: smallint;
   StartSect: smallint;
 end;

 { Sector Type }
 pDukeSector = ^TDukeSector;
 TDukeSector = record
   WallPtr       : smallint;
   WallNum       : smallint;
   CeilingZ      : longint;
   FloorZ        : longint;
   CeilingStat   : smallint;
   FloorStat     : smallint;
   CeilingPic    : smallint;
   CeilingSlope  : smallint;
   CeilingShade  : byte;
   CeilingPal    : byte;
   CeilingxPan   : byte;
   CeilingyPan   : byte;
   FloorPic      : smallint;
   FloorSlope    : smallint;
   FloorShade    : byte;
   FloorPal      : byte;
   FloorxPan     : byte;
   FlooryPan     : byte;
   Visibility    : byte;
   Filler        : byte;
   lotag         :smallint;
   hitag         :smallint;
   extra         :smallint;
end; {size 40}

{ Wall Type }
pDukeWall = ^TDukeWall;
TDukeWall = record
 x         : longint;
 y         : longint;
 point2    : smallint;
 nextwall  : smallint;
 nextsector: smallint;
 cstat     : smallint;
 Pic       : smallint;
 OverPic   : smallint;
 shade     : byte;
 pal       : byte;
 xrepeat   : byte;
 yrepeat   : byte;
 xpanning  : byte;
 ypanning  : byte;
 lotag     : smallint;
 hitag     : smallint;
 extra     : smallint;
 end;

 { Sprite Type }
 pDukeSprite = ^TDukeSprite;
 TDukeSprite = record
   x       : longint;
   y       : longint;
   z       : longint;
   cstat   : smallint;
   pic     : smallint;
   shade   : byte;
   pal     : byte;
   clipdist: byte;
   filler  : byte;
   xrepeat : byte;
   yrepeat : byte;
   xoffset : byte;
   yoffset : byte;
   sectnum : smallint;
   statnum : smallint;
   ang     : smallint;
   owner   : smallint;
   xvel    : smallint;
   yvel    : smallint;
   zvel    : smallint;
   lotag   : smallint;
   hitag   : smallint;
   extra   : smallint;
 end;

{ An object that encapsulates a DN3D level (OOP kicks butt!) }

TDukeLevel = class(TObject)
 Thng: TList;         { list of Sprites}
 Sect: TList;	      { list of Sectors}
 Wall: TList;	      { list of Walls}
 wHead: TDukeHeader;  { the header }
 constructor Create;
 destructor Destroy; override;
 function  AddWall(t: TDukeWall): integer;
 function  GetWall(i: integer): TDukeWall;
 procedure DrawWall(i:Integer; Canvas: TCanvas; clr: TColor);
 function  AddThng(t: TDukeSprite): integer;
 function  GetThng(i: integer): TDukeSprite;
 procedure DelThng(i: integer);
 procedure DrawThng(i:Integer; Canvas: TCanvas; clr: TColor);
 procedure DrawAllThng(Canvas: TCanvas; clr: TColor);
 procedure DrawvTex(i:Integer; Canvas: TCanvas; clr: TColor);
 function  AddSect(t: TDukeSector): integer;
 function  GetSect(i: integer): TDukeSector;
 procedure DelSect(i: integer);
 procedure SethSize(Canvas: TCanvas);
 public
    pSize: integer;
 private
   procedure RemoveWall(i : Integer);
 end;

implementation

constructor TDukeLevel.Create;
begin
  inherited Create;
  Wall := TList.Create;   {create the lists}
  Thng := TList.Create;
  Sect := TList.Create;
end;

destructor TDukeLevel.Destroy;
var i : integer;
begin
 For i := 1 to Wall.Count do RemoveWall(0);  {destroy the lists}
 For i := 1 to Thng.Count do DelThng(0);
 For i := 1 to Sect.Count do DelSect(0);
 Wall.Free;
 Thng.Free;
 Sect.Free;
 inherited Destroy;
end;

procedure TDukeLevel.RemoveWall(i : Integer);
var
  p     : pDukeWall;
begin
 p := Wall.Items[i];
 Wall.Remove(p);
 Dispose(p)
end;

function TDukeLevel.AddWall(t : TDukeWall):integer;
Var p       : pDukeWall;
begin
   New(p);
   p^ := t;
   Result := Wall.Add(p);
end;

function TDukeLevel.GetWall(i : Integer) : tDukeWall;
Var p     : pDukeWall;
begin
  p := Wall.Items[i];
  Result := p^;
end;

Procedure TDukeLevel.DrawWall(i:Integer; Canvas: TCanvas; clr: TColor);
var f, t: tPoint;
    lf, lt: tDukeWall;
    ang, sAmt: Single;
begin
     lf := GetWall(i);
     lt := GetWall(lf.point2);
     if clr = -1 then
       if lf.NextWall = -1 then
             clr := clGray
         else
             clr := clOlive;

     with lf do begin
       x := x div MAPDIV;   {divide by const to get to 16 bit range}
       y := y div MAPDIV
     end;
     with lt do begin
       x := x div MAPDIV;
       y := y div MAPDIV
     end;

     with canvas do begin
       Pen.Color := clr;
       MoveTo(lf.x, lf.y);
       LineTo(lt.x, lt.y);
     end;

end;

function TDukeLevel.AddThng(t : TDukeSprite):integer;
Var p       : pDukeSprite;
begin
 New(p);
 p^ := t;
 Result := Thng.Add(p);
end;

function TDukeLevel.GetThng(i : Integer) : tDukeSprite;
Var p     : pDukeSprite;
begin
  p := Thng.Items[i];
  Result := p^;
end;

procedure TDukeLevel.DelThng(i : Integer);
var
  p     : pDukeSprite;
begin
  p := Thng.Items[i];
  Thng.Remove(p);
  Dispose(p)
end;


procedure TDukeLevel.SethSize(Canvas: TCanvas);
var p: array[0..1] of TPoint;
begin
   { tries to determine logical coords that equal 4 pixels}
   p[0].x := 0; p[0].y := 0;
   p[1].x := 4; p[1].y := 0;
   DPtoLP(Canvas.Handle,p,2);
   pSize := (Abs(p[1].x - p[0].x)) div 2;
end;

Procedure TDukeLevel.DrawThng(i:Integer; Canvas: TCanvas; clr: TColor);
var p: tPoint;
    t: tDukeSprite;
begin
     t := GetThng(i);
     p.x := t.x div MAPDIV;
     p.y := t.y div MAPDIV;

     with canvas do begin
          LPtoDP(Handle,p,1);          {this makes the pixel size uniform?}
          DPtoLP(Handle,p,1);
          Pen.Color := clr;
          Ellipse(p.x-pSize,p.y-pSize, p.x+pSize, p.y+pSize)
     end; {with}
end;

Procedure TDukeLevel.DrawVtex(i:Integer; Canvas: TCanvas; clr: TColor);
var p: tPoint;
    r: tRect;
    w: tDukeWall;
begin
  w := GetWall(i);
  p.x := w.x div MAPDIV;
  p.y := w.y div MAPDIV;

  with canvas do begin
     LPtoDP(Handle,p,1);          {this makes the 'pixel' size uniform? }
     DPtoLP(Handle,p,1);
     Pen.Color := clr;
     Rectangle(p.x-pSize,p.y-pSize, p.x+pSize, p.y+pSize)
  end; {with}
end;


Procedure TDukeLevel.DrawAllThng(Canvas: TCanvas; clr: TColor);
var p: tPoint;
    i: Integer;
begin
     {optimized so color isn't set in a loop}
     Canvas.Pen.Color := clr;
     For i := 0 to Thng.Count-1 do begin
         p.x := self.GetThng(i).x div MAPDIV;
         p.y := self.GetThng(i).y div MAPDIV;
         LPtoDP(canvas.Handle,p,1);
         DPtoLP(canvas.Handle,p,1);
         Canvas.Ellipse(p.x-pSize, p.y-pSize, p.x+pSize, p.y+pSize);
     end;

end;


function TDukeLevel.AddSect(t : TDukeSector):integer;
Var p       : pDukeSector;
begin
 New(p);
 p^ := t;
 Result := Sect.Add(p);
end;

function TDukeLevel.GetSect(i : Integer) : tDukeSector;
Var p     : pDukeSector;
begin
  p := Sect.Items[i];
  Result := p^;
end;

procedure TDukeLevel.DelSect(i : Integer);
var
  p     : pDukeSector;
begin
 p := Sect.Items[i];
 Sect.Remove(p);
 Dispose(p)
end;

{ not needed for demo: you'll need it if you write an editor, though}
{Procedure TDukeLevel.DrawSect(i:Integer; Canvas: TCanvas; clr: TColor);
var lp,cWall: Integer;
    s: TDukeSector;
begin
     s := GetSect(i);
     cWall := s.WallPtr;
     for lp := s.WallPtr to s.WallPtr + s.WallNum - 1 do
       DrawWall(lp,Canvas,Clr);

     if Clr = clRed then
       DrawWall(s.WallPtr,Canvas, clPurple);

end;}

{ not needed for demo: you'll need it if you write an editor, though}
{function TDukeLevel.GetSectFromWall(i:Integer): integer;
var lp: integer;
    Found,Done :boolean;
    s: TDukeSector;
    MaxWall: integer;
begin
 found := false; Done := false;
 lp := 0;
 repeat
   s := GetSect(lp);
   Found := (i >= s.WallPtr) and (i < s.WallPtr + s.WallNum);
   If not Found then begin
     Inc(lp);
     Done := (lp > Sect.Count-1)
   end;
 until Done or Found;
 If found then Result := lp else Result := -1;
end;}

{2D Distance: not needed for demo: you'll need it if you write an editor, though}
{function Dist(x1, x2, y1, y2: Integer): Integer;
var d: Real;
tx, ty: LongInt;
begin
   tx:= LongInt(x1 - x2)*LongInt(x1 - x2);
   ty:= LongInt(y1 - y2)*LongInt(y1 - y2);
   d := Sqrt(tx + ty);
   Dist := Trunc(d);
end;}

{function Sgn(x: Real): integer;
begin
   if x = 0 then Sgn := 0
   else if x < 0 then Sgn := -1
   else Sgn := 1;
end;}

{function BigNum :longint;
var i,r:longint;
begin
r := 1;
for i := 0 to 21 do r := 2*r;
Result := r;
end;}


end.
