unit Mpic1;

interface

uses
  WinTypes, WinProcs,
  SysUtils, Messages, Classes, Graphics, Controls,
  Forms, StdCtrls, DukeType, ExtCtrls;

 {Windows can only handle 16 bit coordinates, and BUILD coordinates
 are all 32 bit.  I divide every coordinate by 16 before scaling and
 displaying the level. }
 const MAPDIV = 16;

type

  TmPic = class(TForm)
    pnInfo: TPanel;
    hScroll: TScrollBar;
    pnFileName: TPanel;
    pnGame: TPanel;
    vScroll: TScrollBar;
    pnLevel: TPanel;
    pncoord: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure KeyPress(Sender: TObject; var Key: Char);
    procedure hScrollScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
    procedure vScrollScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
    procedure MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  private
    pLoadedYet: Boolean;
    pLeft, pRight, pTop, pBottom: integer;
    pMap: TDukeLevel;
    procedure ScaleMap;
    procedure CenterMap(x,y:longint);
  public
    procedure LoadMap(cFile: string);
    procedure DrawMap;
    procedure SizeReset;
  end;

var
  mPic: TmPic;

implementation
uses dCad1;
{$R *.DFM}

procedure TmPic.LoadMap(cFile: String);
var
    iStream: TFileStream;
    i:  longint;
    iNum: smallint;
    lWall: tDukeWall;
    lThng: tDukeSprite;
    lSect: tDukeSector;
    maxv: integer;
begin
     screen.cursor := crHourGlass;
     pnFileName.Caption := lowercase(cFile);

     {open the file, read the header}
     iStream := TFileStream.Create(cFile, fmOpenRead);
     iStream.Read(pMap.wHead, Sizeof(pMap.wHead));

     {read sectors}
     iStream.Read(iNum, Sizeof(iNum));
     For i := 0 to iNum - 1 do begin
       iStream.Read(lSect, Sizeof(lSect));
       pMap.AddSect(lSect);
     end;

     {read walls, set min/max x/y}
     iStream.Read(iNum, Sizeof(iNum));
     For i := 0 to iNum - 1 do begin
       iStream.Read(lWall, Sizeof(lWall));
       pMap.AddWall(lWall);
       If lWall.x div MAPDIV < pLeft   then pLeft := lWall.x div MAPDIV;
       If lWall.x div MAPDIV > pRight  then pRight := lWall.x div MAPDIV;
       If lWall.y div MAPDIV < pTop    then pTop := lWall.y div MAPDIV;
       If lWall.y div MAPDIV > pBottom then pBottom := lWall.y div MAPDIV
     end;

     {read sprites}
     iStream.Read(iNum, Sizeof(iNum));
     For i := 0 to iNum - 1 do begin
       iStream.Read(lThng, Sizeof(lThng));
       pMap.AddThng(lThng);
     end;
     iStream.Free;

     {set scrollbars}
     Dec(pLeft,100); Dec(pTop,100);
     Inc(pRight,100); Inc(pBottom,100);
     with hScroll do begin
       SetParams(pLeft, pLeft, pRight);
       LargeChange := (pRight - pLeft) div 10;
       SmallChange := (pRight - pLeft) div 25
     end;
     with vScroll do begin
       SetParams(pTop, pTop, pBottom);
       LargeChange := (pBottom - pTop) div 10;
       SmallChange := (pBottom - pTop) div 25
     end; {with}
     ScaleMap;

     pLoadedYet := True;

     Invalidate;
     screen.cursor := crDefault;
end;
{--------------}
procedure TmPic.FormCreate(Sender: TObject);
begin
     {move controls into position}
     vScroll.Top := 0;
     vScroll.Left := Self.ClientWidth - vScroll.Width;
     vScroll.Height := Self.ClientHeight - pnInfo.Height;
     hScroll.Width := Self.ClientWidth - hScroll.left - vScroll.Width;

     pLoadedYet := false;
     pnGame.Caption := 'Duke3d';
     pMap := TDukeLevel.Create;
end;

{-------------------------}
procedure TmPic.DrawMap;
var i: integer;

begin

if pLoadedYet then begin

  with canvas do begin
     Pen.Mode := pmCopy;
     Pen.Color := self.Color;
     Brush.Color := self.Color;
     ScaleMap;
     FillRect(Rect(-32767, -32767, 32767, 32767));
   end;

   pMap.SethSize(Canvas);   {determine logical 4 pixel width}

   for i := 0 to pMap.Wall.Count-1 do
     pMap.DrawWall(i, Canvas, -1);

   pMap.DrawAllThng(Canvas, clWhite);

end; {if}

end;

{-------------------------}
procedure TmPic.FormClose(Sender: TObject; var Action: TCloseAction);
begin
     Action := caFree;
     pMap.Free;
     self.release;
end;
{----------------------------}
{ convert hDC to Duke's Scale
{----------------------------}
procedure TmPic.ScaleMap;
var tWidth,tHeight :longint;
    aSize: TSize;
    aPoint: TPoint;
begin
tWidth := (pRight - pLeft);
tHeight := (pbottom - pTop);

with Canvas do begin
 setmapmode(handle,MM_ISOTROPIC);
 setwindowextEx(handle,tWidth,tHeight,@aSize);
 setviewportextEx(handle, ClientWidth-vScroll.Width,ClientHeight-pnInfo.Height,@aSize);
 setwindoworgEx(handle, pleft, ptop, @aPoint);
end;
end;

procedure TmPic.FormPaint(Sender: TObject);
begin if WindowState<> wsMinimized then Drawmap end;

procedure TmPic.FormResize(Sender: TObject);
begin
   {move controls}
   vScroll.Left := ClientWidth - vScroll.Width;
   vScroll.Height := ClientHeight - pnInfo.Height;
   hScroll.Width := ClientWidth - hScroll.left - vScroll.Width;
   Invalidate;
end;


procedure TmPic.MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var p: TPoint;
begin

ScaleMap;
p.x := x; p.y:= y;
DPtoLP(canvas.Handle,p,1);

{Ctrl-LeftClick: center map on clicked spot}
if ((ssCtrl in Shift) and (ssLeft in Shift)) then
    CenterMap(p.x,p.y);

end;

procedure TmPic.KeyPress(Sender: TObject; var Key: Char);
var diffx: longint;
    diffy: longint;

begin
    Case Key of
    '+': begin   {Zoom in, just change boundary coords}
        diffx := (pRight - pLeft) div 8;
        diffy := (pBottom - pTop) div 8;
        pLeft := pLeft + diffx;
        pRight := pRight - diffx;
        pTop := pTop + diffy;
        pBottom := pBottom - diffy;
        HScroll.Position := pLeft;
        vScroll.Position := pTop;
        self.Invalidate
        end;
    '-': begin {Zoom out, just change boundary coords}
        diffx := (pRight - pLeft) div 6;
        diffy := (pBottom - pTop) div 6;
        pLeft := pLeft - diffx;
        pRight := pRight + diffx;
        pTop := pTop - diffy;
        pBottom := pBottom + diffy;
        HScroll.Position := pLeft;
        vScroll.Position := pTop;
        self.Invalidate
        end;
    end;
end;

procedure TMPic.CenterMap(x,y: longint);
var  cx, cy: LongInt;
begin
    {center on spot, just chage boundary coords}
    cx := (pRight - pLeft) div 2;
    cy := (pBottom - pTop) div 2;

    pLeft := x - cx; pRight := x + cx;
    pTop  := y - cy; pBottom := y + cy;
    HScroll.Position := pLeft;
    vScroll.Position := pTop;

    self.Invalidate;
end;


procedure TmPic.hScrollScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
var nDiff: integer;
begin
  if ScrollCode = scEndScroll then begin
     nDiff := pRight - pLeft;
     pLeft := ScrollPos;
     pRight := pLeft + nDiff;
     Invalidate
  end
end;

procedure TmPic.vScrollScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
var nDiff: integer;
begin
  if ScrollCode = scEndScroll then begin
     nDiff := pBottom - pTop;
     pTop := ScrollPos;
     pBottom := pTop + nDiff;
     Invalidate
  end
end;


procedure TMPic.SizeReset;
var i: integer;
   l: tDukeWall;
begin
   {reset map to default view}
   pLeft := 32767; pRight := -32767;
   pTop := 32767; pBottom := -32767;

   For i := 0 to pMap.Wall.Count - 1 do begin
       l := pMap.GetWall(i);
       if l.x < pLeft   then pLeft   := l.X;
       if l.X > pRight  then pRight  := l.X;
       if l.Y < pTop    then pTop    := l.Y;
       if l.Y > pBottom then pBottom := l.Y
   end {for};
   Dec(pLeft,100); Dec(pTop,100);
   Inc(pRight,100); Inc(pBottom,100);
   with hScroll do begin
       SetParams(pLeft, pLeft, pRight);
       LargeChange := (pRight - pLeft) div 10;
       SmallChange := (pRight - pLeft) div 25
   end; {with}
   with vScroll do begin
       SetParams(pTop, pTop, pBottom);
       LargeChange := (pBottom - pTop) div 10;
       SmallChange := (pBottom - pTop) div 25
   end; {with}
   Invalidate;
end;

procedure TmPic.MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  p: TPoint;
begin
{display coords}
ScaleMap;
p.x := x; p.y:= y;
DPtoLP(canvas.Handle,p,1);

pnCoord.Caption := InttoStr(p.x * longint(MAPDIV))+', '+InttoStr(p.y * longint(MAPDIV));

end;

end.
